@@ -1,10 +1,10 @@
-#!/usr/bin/env python2
+#!/usr/bin/python3
 
 import sys
 import os
 import imtools
 import gettext
-import thread
+import _thread as thread
 import subprocess
 import tempfile
 import locale
@@ -20,7 +20,8 @@ import gi
 gi.require_version("Gtk", "3.0")
 from gi.repository import Gio, Gtk, GObject, Gdk, Pango, GLib
 
-sys.path.append('/usr/share/cinnamon/cinnamon-settings/bin')
+import config
+sys.path.append(config.currentPath + "/bin")
 from GSettingsWidgets import *
 
 gettext.install("cinnamon", "/usr/share/locale")
@@ -48,6 +49,11 @@ BACKGROUND_ICONS_SIZE = 100
 BACKGROUND_COLLECTION_TYPE_DIRECTORY = "directory"
 BACKGROUND_COLLECTION_TYPE_XML = "xml"
 
+# even though pickle supports higher protocol versions, we want to version 2 because it's the latest
+# version supported by python2 which (at this time) is still used by older versions of Cinnamon.
+# When those versions are no longer supported, we can consider using a newer version.
+PICKLE_PROTOCOL_VERSION = 2
+
 (STORE_IS_SEPARATOR, STORE_ICON, STORE_NAME, STORE_PATH, STORE_TYPE) = range(5)
 
 # EXIF utility functions (source: http://stackoverflow.com/questions/4228530/pil-thumbnail-is-rotating-my-image)
@@ -59,15 +65,15 @@ def rotate_270(im): return im.transpose(Image.ROTATE_270)
 def transpose(im): return rotate_90(flip_horizontal(im))
 def transverse(im): return rotate_90(flip_vertical(im))
 orientation_funcs = [None,
-                 lambda x: x,
-                 flip_horizontal,
-                 rotate_180,
-                 flip_vertical,
-                 transpose,
-                 rotate_270,
-                 transverse,
-                 rotate_90
-                ]
+                     lambda x: x,
+                     flip_horizontal,
+                     rotate_180,
+                     flip_vertical,
+                     transpose,
+                     rotate_270,
+                     transverse,
+                     rotate_90
+                     ]
 def apply_orientation(im):
     """
     Extract the oritentation EXIF tag from the image, which should be a PIL Image instance,
@@ -103,7 +109,7 @@ class Module:
 
     def on_module_selected(self):
         if not self.loaded:
-            print "Loading Backgrounds module"
+            print("Loading Backgrounds module")
 
             self.sidePage.stack = SettingsStack()
             self.sidePage.add_widget(self.sidePage.stack)
@@ -121,7 +127,7 @@ class Module:
             self.xdg_pictures_directory = os.path.expanduser("~/Pictures")
             xdg_config = os.path.expanduser("~/.config/user-dirs.dirs")
             if os.path.exists(xdg_config) and os.path.exists("/usr/bin/xdg-user-dir"):
-                path = subprocess.check_output(["xdg-user-dir", "PICTURES"]).rstrip("\n")
+                path = subprocess.check_output(["xdg-user-dir", "PICTURES"]).decode("utf-8").rstrip("\n")
                 if os.path.exists(path):
                     self.xdg_pictures_directory = path
 
@@ -340,8 +346,8 @@ class Module:
                 if collection_type == BACKGROUND_COLLECTION_TYPE_XML:
                     self.remove_folder_button.set_sensitive(False)
                 self.update_icon_view(collection_path, collection_type)
-        except Exception, detail:
-            print detail
+        except Exception as detail:
+            print(detail)
 
     def on_row_activated(self, tree, path, column):
         self.folder_tree.set_selection(path)
@@ -413,7 +419,7 @@ class Module:
     def update_folder_list(self):
         path = os.path.expanduser("~/.cinnamon/backgrounds")
         if not os.path.exists(path):
-            rec_mkdir(path)
+            os.makedirs(path, mode=0o755, exist_ok=True)
         path = os.path.expanduser("~/.cinnamon/backgrounds/user-folders.lst")
         if len(self.user_backgrounds) == 0:
             file_data = ""
@@ -495,7 +501,7 @@ class Module:
                                 else:
                                     propAttr = prop.attrib
                                     wpName = prop.text
-                                    locName = self.splitLocaleCode(propAttr.get(locAttrName)) if propAttr.has_key(locAttrName) else ("", "")
+                                    locName = self.splitLocaleCode(propAttr.get(locAttrName)) if locAttrName in propAttr else ("", "")
                                     names.append((locName, wpName))
                         wallpaperData["name"] = self.getLocalWallpaperName(names, loc)
 
@@ -504,9 +510,9 @@ class Module:
                                 wallpaperData["name"] = os.path.basename(wallpaperData["filename"])
                             res.append(wallpaperData)
             return res
-        except Exception, detail:
-            print "Could not parse %s!" % filename
-            print detail
+        except Exception as detail:
+            print("Could not parse %s!" % filename)
+            print(detail)
             return []
 
     def update_secondary_revealer(self, settings, key):
@@ -545,18 +551,23 @@ class PixCache(object):
                     os.mkdir(tmp_cache_path)
                 cache_filename = tmp_cache_path + h + "v2"
 
+                loaded = False
                 if os.path.exists(cache_filename):
                     # load from disk cache
                     try:
-                        with open(cache_filename, "r") as cache_file:
+                        with open(cache_filename, "rb") as cache_file:
                             pix = pickle.load(cache_file)
                         tmp_img = Image.open(BytesIO(pix[0]))
                         pix[0] = self._image_to_pixbuf(tmp_img)
-                    except Exception, detail:
-                        print "Failed to load cache file: %s: %s" % (cache_filename, detail)
-                        pix = None
+                        loaded = True
+                    except Exception as detail:
+                        # most likely either the file is corrupted, or the file was pickled using the
+                        # python2 version of cinnamon settings. Either way, we want to ditch the current
+                        # cache file and generate a new one. This is still backward compatible with older
+                        # Cinnamon versions
+                        os.remove(cache_filename)
 
-                else:
+                if not loaded:
                     if mimetype == "image/svg+xml":
                         # rasterize svg with Gdk-Pixbuf and convert to PIL Image
                         tmp_pix = GdkPixbuf.Pixbuf.new_from_file(filename)
@@ -586,14 +597,14 @@ class PixCache(object):
                     try:
                         png_bytes = BytesIO()
                         img.save(png_bytes, "png")
-                        with open(cache_filename, "w") as cache_file:
-                            pickle.dump([png_bytes.getvalue(), width, height], cache_file, 2)
-                    except Exception, detail:
-                        print "Failed to save cache file: %s: %s" % (cache_filename, detail)
+                        with open(cache_filename, "wb") as cache_file:
+                            pickle.dump([png_bytes.getvalue(), width, height], cache_file, PICKLE_PROTOCOL_VERSION)
+                    except Exception as detail:
+                        print("Failed to save cache file: %s: %s" % (cache_filename, detail))
 
                     pix = [self._image_to_pixbuf(img), width, height]
-            except Exception, detail:
-                print "Failed to convert %s: %s" % (filename, detail)
+            except Exception as detail:
+                print("Failed to convert %s: %s" % (filename, detail))
                 pix = None
             if pix:
                 self._data[filename][size] = pix
@@ -601,10 +612,11 @@ class PixCache(object):
 
     # Convert RGBA PIL Image to Pixbuf
     def _image_to_pixbuf(self, img):
+        [w, h] = img.size
         return GdkPixbuf.Pixbuf.new_from_bytes(GLib.Bytes.new(img.tobytes()),
                                                GdkPixbuf.Colorspace.RGB,
-                                               True, 8, img.width, img.height,
-                                               img.width * 4)
+                                               True, 8, w, h,
+                                               w * 4)
 
 PIX_CACHE = PixCache()
 
@@ -747,8 +759,8 @@ class ThreadedIconView(Gtk.IconView):
                                 if staticNode[-1].tag == "size":
                                     return staticNode[-1].text
                                 return staticNode.text
-            print "Could not find filename in %s" % filename
+            print("Could not find filename in %s" % filename)
             return None
-        except Exception, detail:
-            print "Failed to read filename from %s: %s" % (filename, detail)
+        except Exception as detail:
+            print("Failed to read filename from %s: %s" % (filename, detail))
             return None
